#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);

  int errors = 0, total_tests = 0;
  {  // f_1f_calldet_test0
    last_test = "f_1f_calldet_test0";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 2.0; 
    ShAttrib<1, SH_CONST, float> expected(float(2.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_1f_calldet_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calldet_test1
    last_test = "f_1f_calldet_test1";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = -0.05; 
    ShAttrib<1, SH_CONST, float> expected(float(-0.05));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_1f_calldet_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calldet_test2
    last_test = "f_1f_calldet_test2";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 1; 
    ShAttrib<1, SH_CONST, float> expected(float(1));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_1f_calldet_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calldet_test3
    last_test = "f_2f_calldet_test3";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 0; 
    a[1](0) = 0; a[1](1) = 1; 
    ShAttrib<1, SH_CONST, float> expected(float(1));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_2f_calldet_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calldet_test4
    last_test = "f_2f_calldet_test4";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 2; 
    a[1](0) = 3; a[1](1) = 4; 
    ShAttrib<1, SH_CONST, float> expected(float(-2));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_2f_calldet_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calldet_test5
    last_test = "f_2f_calldet_test5";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1; a[0](1) = 0.2; 
    a[1](0) = 3000; a[1](1) = -4.234; 
    ShAttrib<1, SH_CONST, float> expected(float(-595.766));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_2f_calldet_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calldet_test6
    last_test = "f_3f_calldet_test6";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; 
    ShAttrib<1, SH_CONST, float> expected(float(1.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_3f_calldet_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calldet_test7
    last_test = "f_3f_calldet_test7";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = -1.0; a[0](1) = -2.0; a[0](2) = -3.0; 
    a[1](0) = -3.0; a[1](1) = -4.0; a[1](2) = -5.0; 
    a[2](0) = -5.0; a[2](1) = -6.0; a[2](2) = -7.0; 
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_3f_calldet_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calldet_test8
    last_test = "f_3f_calldet_test8";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 2.0; a[0](2) = 3.0; 
    a[1](0) = 3.0; a[1](1) = 4.0; a[1](2) = 5.0; 
    a[2](0) = 5.0; a[2](1) = 6.0; a[2](2) = 7.0; 
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_3f_calldet_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calldet_test9
    last_test = "f_1f_calldet_test9";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 0; 
    ShAttrib<1, SH_CONST, float> expected(float(0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_1f_calldet_test9", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calldet_test10
    last_test = "f_2f_calldet_test10";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 0; a[0](1) = 0; 
    a[1](0) = 0; a[1](1) = 0; 
    ShAttrib<1, SH_CONST, float> expected(float(0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_2f_calldet_test10", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calldet_test11
    last_test = "f_3f_calldet_test11";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 0.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 0.0; 
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_3f_calldet_test11", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calldet_test12
    last_test = "f_2f_calldet_test12";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1000.0; a[0](1) = 1.3333; 
    a[1](0) = 0.0; a[1](1) = 1.0; 
    ShAttrib<1, SH_CONST, float> expected(float(-1000.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_2f_calldet_test12", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calldet_test13
    last_test = "f_4f_calldet_test13";
    ShMatrix<4, 4, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; a[1](3) = 0.5; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; a[2](3) = 6.7; 
    a[3](0) = 1.0; a[3](1) = -0.3; a[3](2) = -3000; a[3](3) = -1.2; 
    ShAttrib<1, SH_CONST, float> expected(float(10.9));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_4f_calldet_test13", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_5f_calldet_test14
    last_test = "f_5f_calldet_test14";
    ShMatrix<5, 5, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; a[0](4) = -1.34; 
    a[1](0) = 50; a[1](1) = 0.0; a[1](2) = 1.0; a[1](3) = 0.0; a[1](4) = 0.5; 
    a[2](0) = 2.0; a[2](1) = 0.0; a[2](2) = 0.0; a[2](3) = 1.0; a[2](4) = 6.7; 
    a[3](0) = 0.5; a[3](1) = 1.0; a[3](2) = -0.3; a[3](3) = -30; a[3](4) = -1.2; 
    a[4](0) = -0.3; a[4](1) = 0.0; a[4](2) = 1.1; a[4](3) = 3.4; a[4](4) = 90; 
    ShAttrib<1, SH_CONST, float> expected(float(-25911.024));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_5f_calldet_test14", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_6f_calldet_test15
    last_test = "f_6f_calldet_test15";
    ShMatrix<6, 6, SH_TEMP, float> a;
    a[0](0) = 0; a[0](1) = 0.0; a[0](2) = 1.0; a[0](3) = 2.0; a[0](4) = 3.0; a[0](5) = -0.34; 
    a[1](0) = 1; a[1](1) = 5; a[1](2) = 0.0; a[1](3) = 1.0; a[1](4) = 0.0; a[1](5) = 0.5; 
    a[2](0) = 10; a[2](1) = 2.0; a[2](2) = 0.0; a[2](3) = 0.0; a[2](4) = 1.0; a[2](5) = 6.7; 
    a[3](0) = 10; a[3](1) = 0.5; a[3](2) = 1.0; a[3](3) = -0.3; a[3](4) = -0.3; a[3](5) = -1.2; 
    a[4](0) = -0.3; a[4](1) = -0.3; a[4](2) = 0.0; a[4](3) = 1.1; a[4](4) = 3.4; a[4](5) = -0.9; 
    a[5](0) = 1; a[5](1) = 2; a[5](2) = 3; a[5](3) = 4; a[5](4) = 5; a[5](5) = 6; 
    ShAttrib<1, SH_CONST, float> expected(float(1902.6672));

    ShAttrib<1, SH_TEMP, float> out;
    out = det(a);

    try {
      if (test.check("f_6f_calldet_test15", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calladjoint_test0
    last_test = "f_1f_calladjoint_test0";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 2.0; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 2.0; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_1f_calladjoint_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calladjoint_test1
    last_test = "f_1f_calladjoint_test1";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = -0.05; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = -0.05; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_1f_calladjoint_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calladjoint_test2
    last_test = "f_1f_calladjoint_test2";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 1; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 1; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_1f_calladjoint_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calladjoint_test3
    last_test = "f_2f_calladjoint_test3";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 0; 
    a[1](0) = 0; a[1](1) = 1; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 1; expected[0](1) = 0; 
    expected[1](0) = 0; expected[1](1) = 1; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_2f_calladjoint_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calladjoint_test4
    last_test = "f_2f_calladjoint_test4";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 2; 
    a[1](0) = 3; a[1](1) = 4; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 4; expected[0](1) = -2; 
    expected[1](0) = -3; expected[1](1) = 1; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_2f_calladjoint_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calladjoint_test5
    last_test = "f_2f_calladjoint_test5";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1; a[0](1) = 0.2; 
    a[1](0) = 3000; a[1](1) = -4.234; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = -4.234; expected[0](1) = -0.2; 
    expected[1](0) = -3000; expected[1](1) = -1; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_2f_calladjoint_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calladjoint_test6
    last_test = "f_3f_calladjoint_test6";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = -0.0; expected[0](2) = 0.0; 
    expected[1](0) = -0.0; expected[1](1) = 1.0; expected[1](2) = -0.0; 
    expected[2](0) = 0.0; expected[2](1) = -0.0; expected[2](2) = 1.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_3f_calladjoint_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calladjoint_test7
    last_test = "f_3f_calladjoint_test7";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = -1.0; a[0](1) = -2.0; a[0](2) = -3.0; 
    a[1](0) = -3.0; a[1](1) = -4.0; a[1](2) = -5.0; 
    a[2](0) = -5.0; a[2](1) = -6.0; a[2](2) = -7.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = -2.0; expected[0](1) = 4.0; expected[0](2) = -2.0; 
    expected[1](0) = 4.0; expected[1](1) = -8.0; expected[1](2) = 4.0; 
    expected[2](0) = -2.0; expected[2](1) = 4.0; expected[2](2) = -2.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_3f_calladjoint_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calladjoint_test8
    last_test = "f_3f_calladjoint_test8";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 2.0; a[0](2) = 3.0; 
    a[1](0) = 3.0; a[1](1) = 4.0; a[1](2) = 5.0; 
    a[2](0) = 5.0; a[2](1) = 6.0; a[2](2) = 7.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = -2.0; expected[0](1) = 4.0; expected[0](2) = -2.0; 
    expected[1](0) = 4.0; expected[1](1) = -8.0; expected[1](2) = 4.0; 
    expected[2](0) = -2.0; expected[2](1) = 4.0; expected[2](2) = -2.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_3f_calladjoint_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_calladjoint_test9
    last_test = "f_1f_calladjoint_test9";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 0; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 0; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_1f_calladjoint_test9", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calladjoint_test10
    last_test = "f_2f_calladjoint_test10";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 0; a[0](1) = 0; 
    a[1](0) = 0; a[1](1) = 0; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 0; expected[0](1) = 0; 
    expected[1](0) = 0; expected[1](1) = 0; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_2f_calladjoint_test10", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calladjoint_test11
    last_test = "f_3f_calladjoint_test11";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 0.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 0.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 0.0; expected[0](1) = -0.0; expected[0](2) = 0.0; 
    expected[1](0) = -0.0; expected[1](1) = 0.0; expected[1](2) = -0.0; 
    expected[2](0) = 0.0; expected[2](1) = -0.0; expected[2](2) = 0.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_3f_calladjoint_test11", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calladjoint_test12
    last_test = "f_2f_calladjoint_test12";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1000.0; a[0](1) = 1.3333; 
    a[1](0) = 0.0; a[1](1) = 1.0; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = -1.3333; 
    expected[1](0) = -0.0; expected[1](1) = -1000.0; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_2f_calladjoint_test12", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calladjoint_test13
    last_test = "f_4f_calladjoint_test13";
    ShMatrix<4, 4, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; a[1](3) = 0.5; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; a[2](3) = 6.7; 
    a[3](0) = 1.0; a[3](1) = -0.3; a[3](2) = -3000; a[3](3) = -1.2; 
    ShMatrix<4, 4, SH_TEMP, float> expected;
    expected[0](0) = 20098.95; expected[0](1) = -20095.68; expected[0](2) = -7497.9; expected[0](3) = 10.9; 
    expected[1](0) = 0.5; expected[1](1) = 10.4; expected[1](2) = -1.0; expected[1](3) = 0.0; 
    expected[2](0) = 6.7; expected[2](1) = -6.7; expected[2](2) = -2.5; expected[2](3) = -0.0; 
    expected[3](0) = -1.0; expected[3](1) = 1.0; expected[3](2) = 2.0; expected[3](3) = 0.0; 

    ShMatrix<4, 4, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_4f_calladjoint_test13", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_5f_calladjoint_test14
    last_test = "f_5f_calladjoint_test14";
    ShMatrix<5, 5, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; a[0](4) = -1.34; 
    a[1](0) = 50; a[1](1) = 0.0; a[1](2) = 1.0; a[1](3) = 0.0; a[1](4) = 0.5; 
    a[2](0) = 2.0; a[2](1) = 0.0; a[2](2) = 0.0; a[2](3) = 1.0; a[2](4) = 6.7; 
    a[3](0) = 0.5; a[3](1) = 1.0; a[3](2) = -0.3; a[3](3) = -30; a[3](4) = -1.2; 
    a[4](0) = -0.3; a[4](1) = 0.0; a[4](2) = 1.1; a[4](3) = 3.4; a[4](4) = 90; 
    ShMatrix<5, 5, SH_TEMP, float> expected;
    expected[0](0) = 66.67; expected[0](1) = -397.97; expected[0](2) = -2956.236; expected[0](3) = -66.67; expected[0](4) = 222.39; 
    expected[1](0) = -17450.48; expected[1](1) = 10912.2688; expected[1](2) = -267766.512; expected[1](3) = -8460.544; expected[1](4) = 19500.48; 
    expected[2](0) = -3364.55; expected[2](1) = -6040.934; expected[2](2) = 148527.9; expected[2](3) = 3364.55; expected[2](4) = -11028.75; 
    expected[3](0) = -549.41; expected[3](1) = 415.246; expected[3](2) = -10402.812; expected[3](3) = 549.41; expected[3](4) = 771.27; 
    expected[4](0) = 62.1; expected[4](1) = 56.82; expected[4](2) = -1432.2; expected[4](3) = -62.1; expected[4](4) = -181.5; 

    ShMatrix<5, 5, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_5f_calladjoint_test14", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_6f_calladjoint_test15
    last_test = "f_6f_calladjoint_test15";
    ShMatrix<6, 6, SH_TEMP, float> a;
    a[0](0) = 0; a[0](1) = 0.0; a[0](2) = 1.0; a[0](3) = 2.0; a[0](4) = 3.0; a[0](5) = -0.34; 
    a[1](0) = 1; a[1](1) = 5; a[1](2) = 0.0; a[1](3) = 1.0; a[1](4) = 0.0; a[1](5) = 0.5; 
    a[2](0) = 10; a[2](1) = 2.0; a[2](2) = 0.0; a[2](3) = 0.0; a[2](4) = 1.0; a[2](5) = 6.7; 
    a[3](0) = 10; a[3](1) = 0.5; a[3](2) = 1.0; a[3](3) = -0.3; a[3](4) = -0.3; a[3](5) = -1.2; 
    a[4](0) = -0.3; a[4](1) = -0.3; a[4](2) = 0.0; a[4](3) = 1.1; a[4](4) = 3.4; a[4](5) = -0.9; 
    a[5](0) = 1; a[5](1) = 2; a[5](2) = 3; a[5](3) = 4; a[5](4) = 5; a[5](5) = 6; 
    ShMatrix<6, 6, SH_TEMP, float> expected;
    expected[0](0) = 459.446; expected[0](1) = -11.7624; expected[0](2) = 156.4292; expected[0](3) = 45.868; expected[0](4) = -199.652; expected[0](5) = -168.438; 
    expected[1](0) = -1104.715; expected[1](1) = 372.4548; expected[1](2) = -240.025; expected[1](3) = 190.738; expected[1](4) = 614.146; expected[1](5) = 304.659; 
    expected[2](0) = -3145.464; expected[2](1) = -154.6272; expected[2](2) = -1042.3728; expected[2](3) = 956.256; expected[2](4) = 1160.544; expected[2](5) = 1363.9584; 
    expected[3](0) = 5112.519; expected[3](1) = 97.2156; expected[3](2) = 962.0658; expected[3](3) = -916.458; expected[3](4) = -2817.978; expected[3](5) = -1398.687; 
    expected[4](0) = -1736.604; expected[4](1) = -23.4816; expected[4](2) = -275.4168; expected[4](3) = 273.384; expected[4](4) = 1479.768; expected[4](5) = 487.74; 
    expected[5](0) = -96.78; expected[5](1) = -90.12; expected[5](2) = 163.26; expected[5](3) = -166.2; expected[5](4) = -106.2; expected[5](5) = 87.66; 

    ShMatrix<6, 6, SH_TEMP, float> out;
    out = adjoint(a);

    try {
      if (test.check("f_6f_calladjoint_test15", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callinverse_test0
    last_test = "f_1f_callinverse_test0";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 2.0; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 1.0; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_1f_callinverse_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callinverse_test1
    last_test = "f_1f_callinverse_test1";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = -0.05; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 1.0; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_1f_callinverse_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callinverse_test2
    last_test = "f_1f_callinverse_test2";
    ShMatrix<1, 1, SH_TEMP, float> a;
    a[0](0) = 1; 
    ShMatrix<1, 1, SH_TEMP, float> expected;
    expected[0](0) = 1.0; 

    ShMatrix<1, 1, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_1f_callinverse_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callinverse_test3
    last_test = "f_2f_callinverse_test3";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 0; 
    a[1](0) = 0; a[1](1) = 1; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = 0.0; 
    expected[1](0) = 0.0; expected[1](1) = 1.0; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_2f_callinverse_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callinverse_test4
    last_test = "f_2f_callinverse_test4";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 2; 
    a[1](0) = 3; a[1](1) = 4; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = -2.0; expected[0](1) = 1.0; 
    expected[1](0) = 1.5; expected[1](1) = -0.5; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_2f_callinverse_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callinverse_test5
    last_test = "f_2f_callinverse_test5";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1; a[0](1) = 0.2; 
    a[1](0) = 3000; a[1](1) = -4.234; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = 0.00710681710604; expected[0](1) = 0.000335702272369; 
    expected[1](0) = 5.03553408553; expected[1](1) = 0.00167851136184; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_2f_callinverse_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callinverse_test6
    last_test = "f_3f_callinverse_test6";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = -0.0; expected[0](2) = 0.0; 
    expected[1](0) = -0.0; expected[1](1) = 1.0; expected[1](2) = -0.0; 
    expected[2](0) = 0.0; expected[2](1) = -0.0; expected[2](2) = 1.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_3f_callinverse_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callinverse_test7
    last_test = "f_2f_callinverse_test7";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1000.0; a[0](1) = 1.3333; 
    a[1](0) = 0.0; a[1](1) = 1.0; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = -0.001; expected[0](1) = 0.0013333; 
    expected[1](0) = 0.0; expected[1](1) = 1.0; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_2f_callinverse_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callinverse_test8
    last_test = "f_4f_callinverse_test8";
    ShMatrix<4, 4, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; a[1](3) = 0.5; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; a[2](3) = 6.7; 
    a[3](0) = 1.0; a[3](1) = -0.3; a[3](2) = -3000; a[3](3) = -1.2; 
    ShMatrix<4, 4, SH_TEMP, float> expected;
    expected[0](0) = 1843.94036697; expected[0](1) = -1843.64036697; expected[0](2) = -687.880733945; expected[0](3) = 1.0; 
    expected[1](0) = 0.045871559633; expected[1](1) = 0.954128440367; expected[1](2) = -0.0917431192661; expected[1](3) = 0.0; 
    expected[2](0) = 0.614678899083; expected[2](1) = -0.614678899083; expected[2](2) = -0.229357798165; expected[2](3) = -0.0; 
    expected[3](0) = -0.0917431192661; expected[3](1) = 0.0917431192661; expected[3](2) = 0.183486238532; expected[3](3) = 0.0; 

    ShMatrix<4, 4, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_4f_callinverse_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_5f_callinverse_test9
    last_test = "f_5f_callinverse_test9";
    ShMatrix<5, 5, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; a[0](4) = -1.34; 
    a[1](0) = 50; a[1](1) = 0.0; a[1](2) = 1.0; a[1](3) = 0.0; a[1](4) = 0.5; 
    a[2](0) = 2.0; a[2](1) = 0.0; a[2](2) = 0.0; a[2](3) = 1.0; a[2](4) = 6.7; 
    a[3](0) = 0.5; a[3](1) = 1.0; a[3](2) = -0.3; a[3](3) = -30; a[3](4) = -1.2; 
    a[4](0) = -0.3; a[4](1) = 0.0; a[4](2) = 1.1; a[4](3) = 3.4; a[4](4) = 90; 
    ShMatrix<5, 5, SH_TEMP, float> expected;
    expected[0](0) = -0.0025730360946; expected[0](1) = 0.0153590996635; expected[0](2) = 0.114091824391; expected[0](3) = 0.0025730360946; expected[0](4) = -0.0085828333145; 
    expected[1](0) = 0.673477049768; expected[1](1) = -0.421143865252; expected[1](2) = 10.3340768007; expected[1](3) = 0.326522950232; expected[1](4) = -0.752593953832; 
    expected[2](0) = 0.129850136374; expected[2](1) = 0.233141461333; expected[2](2) = -5.73222810492; expected[2](3) = -0.129850136374; expected[2](4) = 0.42563929546; 
    expected[3](0) = 0.0212037162252; expected[3](1) = -0.0160258428999; expected[3](2) = 0.401482087315; expected[3](3) = -0.0212037162252; expected[3](4) = -0.0297660949255; 
    expected[4](0) = -0.00239666328895; expected[4](1) = -0.00219288901898; expected[4](2) = 0.0552737707317; expected[4](3) = 0.00239666328895; expected[4](4) = 0.00700474053052; 

    ShMatrix<5, 5, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_5f_callinverse_test9", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_6f_callinverse_test10
    last_test = "f_6f_callinverse_test10";
    ShMatrix<6, 6, SH_TEMP, float> a;
    a[0](0) = 0; a[0](1) = 0.0; a[0](2) = 1.0; a[0](3) = 2.0; a[0](4) = 3.0; a[0](5) = -0.34; 
    a[1](0) = 1; a[1](1) = 5; a[1](2) = 0.0; a[1](3) = 1.0; a[1](4) = 0.0; a[1](5) = 0.5; 
    a[2](0) = 10; a[2](1) = 2.0; a[2](2) = 0.0; a[2](3) = 0.0; a[2](4) = 1.0; a[2](5) = 6.7; 
    a[3](0) = 10; a[3](1) = 0.5; a[3](2) = 1.0; a[3](3) = -0.3; a[3](4) = -0.3; a[3](5) = -1.2; 
    a[4](0) = -0.3; a[4](1) = -0.3; a[4](2) = 0.0; a[4](3) = 1.1; a[4](4) = 3.4; a[4](5) = -0.9; 
    a[5](0) = 1; a[5](1) = 2; a[5](2) = 3; a[5](3) = 4; a[5](4) = 5; a[5](5) = 6; 
    ShMatrix<6, 6, SH_TEMP, float> expected;
    expected[0](0) = 0.241474704562; expected[0](1) = -0.00618205853341; expected[0](2) = 0.0822157442983; expected[0](3) = 0.0241072111823; expected[0](4) = -0.104932696585; expected[0](5) = -0.0885273052481; 
    expected[1](0) = -0.580613887705; expected[1](1) = 0.195754044638; expected[1](2) = -0.12615185672; expected[1](3) = 0.100247694394; expected[1](4) = 0.322781619402; expected[1](5) = 0.160122064437; 
    expected[2](0) = -1.65318664241; expected[2](1) = -0.0812686527628; expected[2](2) = -0.547848199622; expected[2](3) = 0.502587105091; expected[2](4) = 0.609956381232; expected[2](5) = 0.716866512441; 
    expected[3](0) = 2.68702745283; expected[3](1) = 0.051094379511; expected[3](2) = 0.50564060809; expected[3](3) = -0.4816701523; expected[3](4) = -1.48106720923; expected[3](5) = -0.735119100177; 
    expected[4](0) = -0.912720837359; expected[4](1) = -0.0123414120977; expected[4](2) = -0.144753007778; expected[4](3) = 0.143684612842; expected[4](4) = 0.777733489073; expected[4](5) = 0.256345408172; 
    expected[5](0) = -0.0508654377392; expected[5](1) = -0.0473650883349; expected[5](2) = 0.085805862423; expected[5](3) = -0.0873510617096; expected[5](4) = -0.055816382392; expected[5](5) = 0.0460721664829; 

    ShMatrix<6, 6, SH_TEMP, float> out;
    out = inverse(a);

    try {
      if (test.check("f_6f_callinverse_test10", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test0
    last_test = "f_3f_calltrace_test0";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; 
    ShAttrib<1, SH_CONST, float> expected(float(3.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test1
    last_test = "f_3f_calltrace_test1";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 0.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 0.0; 
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test2
    last_test = "f_3f_calltrace_test2";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = -1.0; a[0](1) = -2.0; a[0](2) = -3.0; 
    a[1](0) = -3.0; a[1](1) = -4.0; a[1](2) = -5.0; 
    a[2](0) = -5.0; a[2](1) = -6.0; a[2](2) = -7.0; 
    ShAttrib<1, SH_CONST, float> expected(float(-12.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test3
    last_test = "f_3f_calltrace_test3";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 2.0; a[0](2) = 3.0; 
    a[1](0) = 3.0; a[1](1) = 4.0; a[1](2) = 5.0; 
    a[2](0) = 5.0; a[2](1) = 6.0; a[2](2) = 7.0; 
    ShAttrib<1, SH_CONST, float> expected(float(12.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calltrace_test4
    last_test = "f_2f_calltrace_test4";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1000.0; a[0](1) = 1.3333; 
    a[1](0) = 0.0; a[1](1) = 1.0; 
    ShAttrib<1, SH_CONST, float> expected(float(-999.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_2f_calltrace_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltrace_test5
    last_test = "f_4f_calltrace_test5";
    ShMatrix<4, 4, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; a[1](3) = 0.5; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; a[2](3) = 6.7; 
    a[3](0) = 1.0; a[3](1) = -0.3; a[3](2) = -3000; a[3](3) = -1.2; 
    ShAttrib<1, SH_CONST, float> expected(float(0.8));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_4f_calltrace_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test6
    last_test = "f_3f_calltrace_test6";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 2; a[0](2) = 3; 
    a[1](0) = 4; a[1](1) = 5; a[1](2) = 6; 
    a[2](0) = -7; a[2](1) = -8; a[2](2) = -9; 
    ShAttrib<1, SH_CONST, float> expected(float(-3));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltrace_test7
    last_test = "f_3f_calltrace_test7";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.1; a[0](1) = 0.1; a[0](2) = 0.1; 
    a[1](0) = -4.5; a[1](1) = -4.5; a[1](2) = -4.5; 
    a[2](0) = 70000; a[2](1) = 70000; a[2](2) = 70000; 
    ShAttrib<1, SH_CONST, float> expected(float(69995.6));

    ShAttrib<1, SH_TEMP, float> out;
    out = trace(a);

    try {
      if (test.check("f_3f_calltrace_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test0
    last_test = "f_3f_calltranspose_test0";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = 0.0; expected[0](2) = 0.0; 
    expected[1](0) = 0.0; expected[1](1) = 1.0; expected[1](2) = 0.0; 
    expected[2](0) = 0.0; expected[2](1) = 0.0; expected[2](2) = 1.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test1
    last_test = "f_3f_calltranspose_test1";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 0.0; a[0](2) = 0.0; 
    a[1](0) = 0.0; a[1](1) = 0.0; a[1](2) = 0.0; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 0.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 0.0; expected[0](1) = 0.0; expected[0](2) = 0.0; 
    expected[1](0) = 0.0; expected[1](1) = 0.0; expected[1](2) = 0.0; 
    expected[2](0) = 0.0; expected[2](1) = 0.0; expected[2](2) = 0.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test2
    last_test = "f_3f_calltranspose_test2";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = -1.0; a[0](1) = -2.0; a[0](2) = -3.0; 
    a[1](0) = -3.0; a[1](1) = -4.0; a[1](2) = -5.0; 
    a[2](0) = -5.0; a[2](1) = -6.0; a[2](2) = -7.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = -1.0; expected[0](1) = -3.0; expected[0](2) = -5.0; 
    expected[1](0) = -2.0; expected[1](1) = -4.0; expected[1](2) = -6.0; 
    expected[2](0) = -3.0; expected[2](1) = -5.0; expected[2](2) = -7.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test3
    last_test = "f_3f_calltranspose_test3";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1.0; a[0](1) = 2.0; a[0](2) = 3.0; 
    a[1](0) = 3.0; a[1](1) = 4.0; a[1](2) = 5.0; 
    a[2](0) = 5.0; a[2](1) = 6.0; a[2](2) = 7.0; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 1.0; expected[0](1) = 3.0; expected[0](2) = 5.0; 
    expected[1](0) = 2.0; expected[1](1) = 4.0; expected[1](2) = 6.0; 
    expected[2](0) = 3.0; expected[2](1) = 5.0; expected[2](2) = 7.0; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_calltranspose_test4
    last_test = "f_2f_calltranspose_test4";
    ShMatrix<2, 2, SH_TEMP, float> a;
    a[0](0) = -1000.0; a[0](1) = 1.3333; 
    a[1](0) = 0.0; a[1](1) = 1.0; 
    ShMatrix<2, 2, SH_TEMP, float> expected;
    expected[0](0) = -1000.0; expected[0](1) = 0.0; 
    expected[1](0) = 1.3333; expected[1](1) = 1.0; 

    ShMatrix<2, 2, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_2f_calltranspose_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltranspose_test5
    last_test = "f_4f_calltranspose_test5";
    ShMatrix<4, 4, SH_TEMP, float> a;
    a[0](0) = 0.0; a[0](1) = 1.0; a[0](2) = 2.0; a[0](3) = 3.0; 
    a[1](0) = 0.0; a[1](1) = 1.0; a[1](2) = 0.0; a[1](3) = 0.5; 
    a[2](0) = 0.0; a[2](1) = 0.0; a[2](2) = 1.0; a[2](3) = 6.7; 
    a[3](0) = 1.0; a[3](1) = -0.3; a[3](2) = -3000; a[3](3) = -1.2; 
    ShMatrix<4, 4, SH_TEMP, float> expected;
    expected[0](0) = 0.0; expected[0](1) = 0.0; expected[0](2) = 0.0; expected[0](3) = 1.0; 
    expected[1](0) = 1.0; expected[1](1) = 1.0; expected[1](2) = 0.0; expected[1](3) = -0.3; 
    expected[2](0) = 2.0; expected[2](1) = 0.0; expected[2](2) = 1.0; expected[2](3) = -3000; 
    expected[3](0) = 3.0; expected[3](1) = 0.5; expected[3](2) = 6.7; expected[3](3) = -1.2; 

    ShMatrix<4, 4, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_4f_calltranspose_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test6
    last_test = "f_3f_calltranspose_test6";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 1; a[0](1) = 2; a[0](2) = 3; 
    a[1](0) = 4; a[1](1) = 5; a[1](2) = 6; 
    a[2](0) = -7; a[2](1) = -8; a[2](2) = -9; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 1; expected[0](1) = 4; expected[0](2) = -7; 
    expected[1](0) = 2; expected[1](1) = 5; expected[1](2) = -8; 
    expected[2](0) = 3; expected[2](1) = 6; expected[2](2) = -9; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltranspose_test7
    last_test = "f_3f_calltranspose_test7";
    ShMatrix<3, 3, SH_TEMP, float> a;
    a[0](0) = 0.1; a[0](1) = 0.1; a[0](2) = 0.1; 
    a[1](0) = -4.5; a[1](1) = -4.5; a[1](2) = -4.5; 
    a[2](0) = 70000; a[2](1) = 70000; a[2](2) = 70000; 
    ShMatrix<3, 3, SH_TEMP, float> expected;
    expected[0](0) = 0.1; expected[0](1) = -4.5; expected[0](2) = 70000; 
    expected[1](0) = 0.1; expected[1](1) = -4.5; expected[1](2) = 70000; 
    expected[2](0) = 0.1; expected[2](1) = -4.5; expected[2](2) = 70000; 

    ShMatrix<3, 3, SH_TEMP, float> out;
    out = transpose(a);

    try {
      if (test.check("f_3f_calltranspose_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

