#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);
  if (test.backend() == "cc") return 77;

  int errors = 0, total_tests = 0;
  ShNoMIPFilter<ShTexture2D<ShColor3f> > test2d_tex;
  {
    ShImage image;
    ShUtil::load_PNG(image, "tex.png");
    test2d_tex.size(image.width(), image.height());
    test2d_tex.memory(image.memory());
  }


  ShNoMIPFilter<ShWrapRepeat<ShTexture2D<ShColor3f> > > repeat_tex;
  {
    ShImage image;
    ShUtil::load_PNG(image, "tex.png");
    repeat_tex.size(image.width(), image.height());
    repeat_tex.memory(image.memory());
  }


  ShNoMIPFilter<ShTexture2D<ShColor3fub> > clamp_tex(16,16);
  ShPointer<ShDataVariant<ShFracUByte, SH_MEM> > clamp_tex_memdata = new ShDataVariant<ShFracUByte, SH_MEM>((3 * 16 * 16));
  {
    ShPointer<ShDataVariant<ShFracUByte, SH_HOST> > data = new ShDataVariant<ShFracUByte, SH_HOST>((3 * 16 * 16));
    int index = 0;
    for(int j = 0; j < 16; ++j)
    for(int i = 0; i < 16; ++i)
    for(int elem = 0; elem < 3; ++elem, ++index)
    {
       (*data)[index] = (i - 2 * j + 4 * elem) / 2;
    }
    clamp_tex_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((3 * 16 * 16) * clamp_tex_memdata->datasize(), clamp_tex_memdata->array(), SH_FUBYTE);
    clamp_tex.memory(hostmem);
  }


  ShNoMIPFilter<ShTexture2D<ShColor4fui> > frac_uint2d_tex(8,8);
  ShPointer<ShDataVariant<ShFracUInt, SH_MEM> > frac_uint2d_tex_memdata = new ShDataVariant<ShFracUInt, SH_MEM>((4 * 8 * 8));
  {
    ShPointer<ShDataVariant<ShFracUInt, SH_HOST> > data = new ShDataVariant<ShFracUInt, SH_HOST>((4 * 8 * 8));
    int index = 0;
    for(int j = 0; j < 8; ++j)
    for(int i = 0; i < 8; ++i)
    for(int elem = 0; elem < 4; ++elem, ++index)
    {
       (*data)[index] = (i + 3 * j + 7 * elem) / 100.0;
    }
    frac_uint2d_tex_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((4 * 8 * 8) * frac_uint2d_tex_memdata->datasize(), frac_uint2d_tex_memdata->array(), SH_FUINT);
    frac_uint2d_tex.memory(hostmem);
  }


  ShNoMIPFilter<ShTexture2D<ShColor3fb> > frac_byte2d_tex(8,8);
  ShPointer<ShDataVariant<ShFracByte, SH_MEM> > frac_byte2d_tex_memdata = new ShDataVariant<ShFracByte, SH_MEM>((3 * 8 * 8));
  {
    ShPointer<ShDataVariant<ShFracByte, SH_HOST> > data = new ShDataVariant<ShFracByte, SH_HOST>((3 * 8 * 8));
    int index = 0;
    for(int j = 0; j < 8; ++j)
    for(int i = 0; i < 8; ++i)
    for(int elem = 0; elem < 3; ++elem, ++index)
    {
       (*data)[index] = (-i + j + elem - 1) / 10.0;
    }
    frac_byte2d_tex_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((3 * 8 * 8) * frac_byte2d_tex_memdata->datasize(), frac_byte2d_tex_memdata->array(), SH_FBYTE);
    frac_byte2d_tex.memory(hostmem);
  }


  ShNoMIPFilter<ShTexture2D<ShColor3h> > half2d_tex(16,16);
  ShPointer<ShDataVariant<ShHalf, SH_MEM> > half2d_tex_memdata = new ShDataVariant<ShHalf, SH_MEM>((3 * 16 * 16));
  {
    ShPointer<ShDataVariant<ShHalf, SH_HOST> > data = new ShDataVariant<ShHalf, SH_HOST>((3 * 16 * 16));
    int index = 0;
    for(int j = 0; j < 16; ++j)
    for(int i = 0; i < 16; ++i)
    for(int elem = 0; elem < 3; ++elem, ++index)
    {
       (*data)[index] = 100 * (i - 8) + 10 * (j - 8) + elem;
    }
    half2d_tex_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((3 * 16 * 16) * half2d_tex_memdata->datasize(), half2d_tex_memdata->array(), SH_HALF);
    half2d_tex.memory(hostmem);
  }


  ShNoMIPFilter<ShTextureRect<ShColor3h> > half_rect_tex(16,16);
  ShPointer<ShDataVariant<ShHalf, SH_MEM> > half_rect_tex_memdata = new ShDataVariant<ShHalf, SH_MEM>((3 * 16 * 16));
  {
    ShPointer<ShDataVariant<ShHalf, SH_HOST> > data = new ShDataVariant<ShHalf, SH_HOST>((3 * 16 * 16));
    int index = 0;
    for(int j = 0; j < 16; ++j)
    for(int i = 0; i < 16; ++i)
    for(int elem = 0; elem < 3; ++elem, ++index)
    {
       (*data)[index] = 100 * (i - 8) + 10 * (j - 8) + elem;
    }
    half_rect_tex_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((3 * 16 * 16) * half_rect_tex_memdata->datasize(), half_rect_tex_memdata->array(), SH_HALF);
    half_rect_tex.memory(hostmem);
  }


  ShArrayRect<ShColor3f> float_array(16,16);
  ShPointer<ShDataVariant<float, SH_MEM> > float_array_memdata = new ShDataVariant<float, SH_MEM>((3 * 16 * 16));
  {
    ShPointer<ShDataVariant<float, SH_HOST> > data = new ShDataVariant<float, SH_HOST>((3 * 16 * 16));
    int index = 0;
    for(int j = 0; j < 16; ++j)
    for(int i = 0; i < 16; ++i)
    for(int elem = 0; elem < 3; ++elem, ++index)
    {
       (*data)[index] = i + 3 * j;
    }
    float_array_memdata->set(data);
    ShPointer<ShHostMemory> hostmem = new ShHostMemory((3 * 16 * 16) * float_array_memdata->datasize(), float_array_memdata->array(), SH_FLOAT);
    float_array.memory(hostmem);
  }


  ShProgram tex_a_f_2f_calltest2dtex_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = test2d_tex(a);
  } SH_END;

  tex_a_f_2f_calltest2dtex_test0.name("tex_a_f_2f_calltest2dtex_test0");
  last_test = "tex_a_f_2f_calltest2dtex_test0";
  {  // f_2f_calltest2dtex_test0
    ShAttrib<2, SH_CONST, float> a(float(0.0), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(0.0), float(0.0));

    try {
      if (test.run(tex_a_f_2f_calltest2dtex_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_calltest2dtex_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = test2d_tex(a);
  } SH_END;

  tex_a_f_2f_calltest2dtex_test1.name("tex_a_f_2f_calltest2dtex_test1");
  last_test = "tex_a_f_2f_calltest2dtex_test1";
  {  // f_2f_calltest2dtex_test1
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(1.0), float(0.0));

    try {
      if (test.run(tex_a_f_2f_calltest2dtex_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_calltest2dtex_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = test2d_tex(a);
  } SH_END;

  tex_a_f_2f_calltest2dtex_test2.name("tex_a_f_2f_calltest2dtex_test2");
  last_test = "tex_a_f_2f_calltest2dtex_test2";
  {  // f_2f_calltest2dtex_test2
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(0.0), float(1.0));

    try {
      if (test.run(tex_a_f_2f_calltest2dtex_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_calltest2dtex_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = test2d_tex(a);
  } SH_END;

  tex_a_f_2f_calltest2dtex_test3.name("tex_a_f_2f_calltest2dtex_test3");
  last_test = "tex_a_f_2f_calltest2dtex_test3";
  {  // f_2f_calltest2dtex_test3
    ShAttrib<2, SH_CONST, float> a(float(0.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.5), float(0.5), float(0.5));

    try {
      if (test.run(tex_a_f_2f_calltest2dtex_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callrepeattex_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = repeat_tex(a);
  } SH_END;

  tex_a_f_2f_callrepeattex_test4.name("tex_a_f_2f_callrepeattex_test4");
  last_test = "tex_a_f_2f_callrepeattex_test4";
  {  // f_2f_callrepeattex_test4
    ShAttrib<2, SH_CONST, float> a(float(0.125), float(0.125));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(0.0), float(0.0));

    try {
      if (test.run(tex_a_f_2f_callrepeattex_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callrepeattex_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = repeat_tex(a);
  } SH_END;

  tex_a_f_2f_callrepeattex_test5.name("tex_a_f_2f_callrepeattex_test5");
  last_test = "tex_a_f_2f_callrepeattex_test5";
  {  // f_2f_callrepeattex_test5
    ShAttrib<2, SH_CONST, float> a(float(1.125), float(-0.875));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(0.0), float(0.0));

    try {
      if (test.run(tex_a_f_2f_callrepeattex_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_i_2i_callclamptex_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, int> a;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = clamp_tex(a);
  } SH_END;

  tex_a_i_2i_callclamptex_test6.name("tex_a_i_2i_callclamptex_test6");
  last_test = "tex_a_i_2i_callclamptex_test6";
  {  // i_2i_callclamptex_test6
    ShAttrib<2, SH_CONST, int> a(int(0), int(0));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(1), int(1));

    try {
      if (test.run(tex_a_i_2i_callclamptex_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callclamptex_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = clamp_tex(a);
  } SH_END;

  tex_a_f_2f_callclamptex_test7.name("tex_a_f_2f_callclamptex_test7");
  last_test = "tex_a_f_2f_callclamptex_test7";
  {  // f_2f_callclamptex_test7
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(0), float(0), float(0));

    try {
      if (test.run(tex_a_f_2f_callclamptex_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callfracuint2dtex_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = frac_uint2d_tex(a);
  } SH_END;

  tex_a_f_2f_callfracuint2dtex_test8.name("tex_a_f_2f_callfracuint2dtex_test8");
  last_test = "tex_a_f_2f_callfracuint2dtex_test8";
  {  // f_2f_callfracuint2dtex_test8
    ShAttrib<2, SH_CONST, float> a(float(0), float(0));
    ShAttrib<4, SH_CONST, float> expected(float(0), float(0.07), float(0.14), float(0.21));

    try {
      if (test.run(tex_a_f_2f_callfracuint2dtex_test8, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callfracuint2dtex_test9 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = frac_uint2d_tex(a);
  } SH_END;

  tex_a_f_2f_callfracuint2dtex_test9.name("tex_a_f_2f_callfracuint2dtex_test9");
  last_test = "tex_a_f_2f_callfracuint2dtex_test9";
  {  // f_2f_callfracuint2dtex_test9
    ShAttrib<2, SH_CONST, float> a(float(1), float(0));
    ShAttrib<4, SH_CONST, float> expected(float(0.07), float(0.14), float(0.21), float(0.28));

    try {
      if (test.run(tex_a_f_2f_callfracuint2dtex_test9, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callfracuint2dtex_test10 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = frac_uint2d_tex(a);
  } SH_END;

  tex_a_f_2f_callfracuint2dtex_test10.name("tex_a_f_2f_callfracuint2dtex_test10");
  last_test = "tex_a_f_2f_callfracuint2dtex_test10";
  {  // f_2f_callfracuint2dtex_test10
    ShAttrib<2, SH_CONST, float> a(float(0), float(1));
    ShAttrib<4, SH_CONST, float> expected(float(0.21), float(0.28), float(0.35), float(0.42));

    try {
      if (test.run(tex_a_f_2f_callfracuint2dtex_test10, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callfracuint2dtex_test11 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = frac_uint2d_tex(a);
  } SH_END;

  tex_a_f_2f_callfracuint2dtex_test11.name("tex_a_f_2f_callfracuint2dtex_test11");
  last_test = "tex_a_f_2f_callfracuint2dtex_test11";
  {  // f_2f_callfracuint2dtex_test11
    ShAttrib<2, SH_CONST, float> a(float(1), float(1));
    ShAttrib<4, SH_CONST, float> expected(float(0.28), float(0.35), float(0.42), float(0.49));

    try {
      if (test.run(tex_a_f_2f_callfracuint2dtex_test11, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callfracuint2dtex_test12 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = frac_uint2d_tex(a);
  } SH_END;

  tex_a_f_2f_callfracuint2dtex_test12.name("tex_a_f_2f_callfracuint2dtex_test12");
  last_test = "tex_a_f_2f_callfracuint2dtex_test12";
  {  // f_2f_callfracuint2dtex_test12
    ShAttrib<2, SH_CONST, float> a(float(0.571428571429), float(0.714285714286));
    ShAttrib<4, SH_CONST, float> expected(float(0.19), float(0.26), float(0.33), float(0.4));

    try {
      if (test.run(tex_a_f_2f_callfracuint2dtex_test12, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_i_2i_callhalf2dtex_test13 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, int> a;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = half2d_tex(a);
  } SH_END;

  tex_a_i_2i_callhalf2dtex_test13.name("tex_a_i_2i_callhalf2dtex_test13");
  last_test = "tex_a_i_2i_callhalf2dtex_test13";
  {  // i_2i_callhalf2dtex_test13
    ShAttrib<2, SH_CONST, int> a(int(0), int(0));
    ShAttrib<3, SH_CONST, int> expected(int(-880), int(-879), int(-878));

    try {
      if (test.run(tex_a_i_2i_callhalf2dtex_test13, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callhalf2dtex_test14 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half2d_tex(a);
  } SH_END;

  tex_a_f_2f_callhalf2dtex_test14.name("tex_a_f_2f_callhalf2dtex_test14");
  last_test = "tex_a_f_2f_callhalf2dtex_test14";
  {  // f_2f_callhalf2dtex_test14
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(770), float(771), float(772));

    try {
      if (test.run(tex_a_f_2f_callhalf2dtex_test14, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callhalf2dtex_test15 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half2d_tex(a);
  } SH_END;

  tex_a_f_2f_callhalf2dtex_test15.name("tex_a_f_2f_callhalf2dtex_test15");
  last_test = "tex_a_f_2f_callhalf2dtex_test15";
  {  // f_2f_callhalf2dtex_test15
    ShAttrib<2, SH_CONST, float> a(float(0.53125), float(0.59375));
    ShAttrib<3, SH_CONST, float> expected(float(10), float(11), float(12));

    try {
      if (test.run(tex_a_f_2f_callhalf2dtex_test15, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_callhalf2dtex_test16 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half2d_tex(a);
  } SH_END;

  tex_a_f_2f_callhalf2dtex_test16.name("tex_a_f_2f_callhalf2dtex_test16");
  last_test = "tex_a_f_2f_callhalf2dtex_test16";
  {  // f_2f_callhalf2dtex_test16
    ShAttrib<2, SH_CONST, float> a(float(-2), float(2));
    ShAttrib<3, SH_CONST, float> expected(float(-730), float(-729), float(-728));

    try {
      if (test.run(tex_a_f_2f_callhalf2dtex_test16, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_i_2i_lookuphalfrecttex_test17 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, int> a;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = half_rect_tex[a];
  } SH_END;

  tex_a_i_2i_lookuphalfrecttex_test17.name("tex_a_i_2i_lookuphalfrecttex_test17");
  last_test = "tex_a_i_2i_lookuphalfrecttex_test17";
  {  // i_2i_lookuphalfrecttex_test17
    ShAttrib<2, SH_CONST, int> a(int(0), int(0));
    ShAttrib<3, SH_CONST, int> expected(int(-880), int(-879), int(-878));

    try {
      if (test.run(tex_a_i_2i_lookuphalfrecttex_test17, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_lookuphalfrecttex_test18 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half_rect_tex[a];
  } SH_END;

  tex_a_f_2f_lookuphalfrecttex_test18.name("tex_a_f_2f_lookuphalfrecttex_test18");
  last_test = "tex_a_f_2f_lookuphalfrecttex_test18";
  {  // f_2f_lookuphalfrecttex_test18
    ShAttrib<2, SH_CONST, float> a(float(15.5), float(15.5));
    ShAttrib<3, SH_CONST, float> expected(float(770), float(771), float(772));

    try {
      if (test.run(tex_a_f_2f_lookuphalfrecttex_test18, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_lookuphalfrecttex_test19 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half_rect_tex[a];
  } SH_END;

  tex_a_f_2f_lookuphalfrecttex_test19.name("tex_a_f_2f_lookuphalfrecttex_test19");
  last_test = "tex_a_f_2f_lookuphalfrecttex_test19";
  {  // f_2f_lookuphalfrecttex_test19
    ShAttrib<2, SH_CONST, float> a(float(8.5), float(9.5));
    ShAttrib<3, SH_CONST, float> expected(float(10), float(11), float(12));

    try {
      if (test.run(tex_a_f_2f_lookuphalfrecttex_test19, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_lookuphalfrecttex_test20 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half_rect_tex[a];
  } SH_END;

  tex_a_f_2f_lookuphalfrecttex_test20.name("tex_a_f_2f_lookuphalfrecttex_test20");
  last_test = "tex_a_f_2f_lookuphalfrecttex_test20";
  {  // f_2f_lookuphalfrecttex_test20
    ShAttrib<2, SH_CONST, float> a(float(-20), float(20));
    ShAttrib<3, SH_CONST, float> expected(float(-730), float(-729), float(-728));

    try {
      if (test.run(tex_a_f_2f_lookuphalfrecttex_test20, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_f_2f_lookuphalfrecttex_test21 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = half_rect_tex[a];
  } SH_END;

  tex_a_f_2f_lookuphalfrecttex_test21.name("tex_a_f_2f_lookuphalfrecttex_test21");
  last_test = "tex_a_f_2f_lookuphalfrecttex_test21";
  {  // f_2f_lookuphalfrecttex_test21
    ShAttrib<2, SH_CONST, float> a(float(-20), float(20));
    ShAttrib<3, SH_CONST, float> expected(float(-730), float(-729), float(-728));

    try {
      if (test.run(tex_a_f_2f_lookuphalfrecttex_test21, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tex_a_i_2i_lookupfloatarray_test22 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, int> a;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = float_array[a];
  } SH_END;

  tex_a_i_2i_lookupfloatarray_test22.name("tex_a_i_2i_lookupfloatarray_test22");
  last_test = "tex_a_i_2i_lookupfloatarray_test22";
  {  // i_2i_lookupfloatarray_test22
    ShAttrib<2, SH_CONST, int> a(int(8), int(8));
    ShAttrib<3, SH_CONST, int> expected(int(32), int(32), int(32));

    try {
      if (test.run(tex_a_i_2i_lookupfloatarray_test22, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

